#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis {
  [StorableClass]
  [Item("TradingProblemData", "Represents an item containing all data defining a trading problem.")]
  public sealed class TradingProblemData : DataAnalysisProblemData, ITradingProblemData {
    private const string PriceVariableParameterName = "PriceVariable";
    private const string TransactionCostsParameterName = "TransactionCosts";

    #region default data
    private static double[,] audInUsd = new double[,] {
    { 0.0000},
    { 0.0003},
    {-0.0004},
    { 0.0000},
    { 0.0003},
    { 0.0003},
    {-0.0005},
    { 0.0001},
    { 0.0004},
    { 0.0001},
    {-0.0005},
    {-0.0014},
    {-0.0002},
    { 0.0000},
    { 0.0011},
    { 0.0024},
    {-0.0022},
    {-0.0035},
    { 0.0003},
    { 0.0005},
    { 0.0014},
    {-0.0003},
    { 0.0001},
    { 0.0001},
    { 0.0001},
    { 0.0008},
    {-0.0004},
    { 0.0012},
    { 0.0001},
    {-0.0001},
    { 0.0004},
    {-0.0004},
    {-0.0007},
    {-0.0007},
    {-0.0011},
    { 0.0001},
    {-0.0003},
    {-0.0009},
    { 0.0001},
    {-0.0009},
    { 0.0006},
    { 0.0006},
    {-0.0006},
    { 0.0015},
    { 0.0003},
    {-0.0005},
    {-0.0010},
    {-0.0005},
    { 0.0002},
    {-0.0005},
    {-0.0002},
    {-0.0010},
    { 0.0015},
    { 0.0001},
    {-0.0010},
    {-0.0003},
    { 0.0004},
    {-0.0022},
    { 0.0008},
    {-0.0007},
    { 0.0004},
    { 0.0036},
    {-0.0005},
    { 0.0002},
    { 0.0009},
    {-0.0002},
    { 0.0004},
    { 0.0012},
    {-0.0005},
    { 0.0004},
    {-0.0010},
    {-0.0006},
    {-0.0028},
    { 0.0007},
    { 0.0009},
    { 0.0005},
    {-0.0001},
    { 0.0001},
    {-0.0011},
    { 0.0004},
    { 0.0007},
    {-0.0004},
    { 0.0005},
    {-0.0008},
    {-0.0005},
    {-0.0011},
    { 0.0004},
    {-0.0008},
    { 0.0016},
    { 0.0008},
    {-0.0002},
    { 0.0000},
    {-0.0015},
    { 0.0002},
    {-0.0008},
    {-0.0005},
    {-0.0001},
    { 0.0002},
    { 0.0038},
    { 0.0007},
    {-0.0001},
    { 0.0000},
    { 0.0015},
    { 0.0000},
    { 0.0009},
    {-0.0008},
    {-0.0001},
    {-0.0006},
    { 0.0019},
    {-0.0002},
    { 0.0002},
    {-0.0017},
    {-0.0003},
    {-0.0004},
    {-0.0017},
    { 0.0003},
    { 0.0000},
    { 0.0001},
    { 0.0003},
    { 0.0006},
    {-0.0001},
    {-0.0009},
    { 0.0007},
    { 0.0004},
    { 0.0002},
    { 0.0001},
    {-0.0001},
    {-0.0013},
    { 0.0009},
    { 0.0003},
    { 0.0004},
    { 0.0001},
    { 0.0001},
    { 0.0001},
    { 0.0007},
    { 0.0013},
    { 0.0029},
    {-0.0011},
    { 0.0003},
    { 0.0001},
    { 0.0003},
    {-0.0005},
    { 0.0003},
    { 0.0004},
    { 0.0008},
    {-0.0006},
    { 0.0005},
    {-0.0001},
    { 0.0000},
    { 0.0003},
    { 0.0013},
    { 0.0005},
    { 0.0002},
    { 0.0005},
    {-0.0005},
    { 0.0012},
    {-0.0006},
    { 0.0001},
    {-0.0012},
    { 0.0015},
    {-0.0005},
    {-0.0005},
    { 0.0012},
    {-0.0006},
    {-0.0001},
    {-0.0002},
    { 0.0000},
    { 0.0003},
    {-0.0008},
    { 0.0010},
    { 0.0003},
    {-0.0006},
    { 0.0001},
    {-0.0001},
    { 0.0014},
    {-0.0011},
    {-0.0002},
    {-0.0001},
    { 0.0005},
    { 0.0006},
    {-0.0018},
    {-0.0007},
    {-0.0001},
    {-0.0001},
    { 0.0002},
    {-0.0008},
    { 0.0027},
    { 0.0006},
    {-0.0005},
    {-0.0003},
    {-0.0003},
    {-0.0001},
    { 0.0005},
    { 0.0003},
    { 0.0004},
    { 0.0013},
    { 0.0006},
    { 0.0000},
    { 0.0011},
    { 0.0003},
    {-0.0008},
    {-0.0003},
    { 0.0005},
    {-0.0004},
    {-0.0015},
    {-0.0006},
    { 0.0008},
    { 0.0016},
    {-0.0009},
    { 0.0007},
    {-0.0009},
    {-0.0005},
    { 0.0001},
    { 0.0005},
    { 0.0002},
    {-0.0011},
    { 0.0006},
    {-0.0016},
    {-0.0013},
    {-0.0006},
    {-0.0004},
    {-0.0002},
    {-0.0006},
    { 0.0014},
    { 0.0002},
    {-0.0006},
    {-0.0007},
    {-0.0001},
    {-0.0010},
    {-0.0012},
    { 0.0009},
    {-0.0009},
    {-0.0006},
    { 0.0000},
    { 0.0014},
    {-0.0011},
    {-0.0002},
    {-0.0012},
    {-0.0003},
    { 0.0003},
    {-0.0010},
    {-0.0008},
    { 0.0001},
    {-0.0005},
    {-0.0011},
    { 0.0006},
    {-0.0008},
    { 0.0012},
    { 0.0007},
    {-0.0009},
    { 0.0005},
    { 0.0017},
    { 0.0003},
    {-0.0011},
    { 0.0034},
    {-0.0017},
    {-0.0006},
    {-0.0008},
    { 0.0004},
    {-0.0005},
    {-0.0006},
    { 0.0001},
    { 0.0009},
    {-0.0036},
    {-0.0010},
    { 0.0005},
    { 0.0000},
    { 0.0003},
    {-0.0007},
    { 0.0021},
    { 0.0003},
    { 0.0005},
    { 0.0003},
    {-0.0005},
    {-0.0001},
    {-0.0011},
    {-0.0009},
    { 0.0007},
    { 0.0003},
    { 0.0006},
    { 0.0001},
    {-0.0002},
    { 0.0006},
    { 0.0005},
    { 0.0005},
    {-0.0008},
    {-0.0008},
    { 0.0005},
    { 0.0006},
    {-0.0007},
    { 0.0003},
    { 0.0005},
    { 0.0000},
    { 0.0003},
    { 0.0010},
    {-0.0018},
    { 0.0003},
    {-0.0001},
    { 0.0008},
    {-0.0015},
    {-0.0008},
    { 0.0032},
    { 0.0009},
    { 0.0005},
    {-0.0014},
    {-0.0003},
    { 0.0029},
    { 0.0002},
    {-0.0002},
    {-0.0002},
    { 0.0005},
    { 0.0008},
    {-0.0007},
    { 0.0014},
    {-0.0004},
    {-0.0003},
    { 0.0004},
    {-0.0001},
    {-0.0007},
    { 0.0002},
    { 0.0002},
    { 0.0005},
    { 0.0012},
    {-0.0018},
    { 0.0014},
    { 0.0004},
    { 0.0012},
    { 0.0002},
    { 0.0009},
    {-0.0007},
    { 0.0002},
    { 0.0000},
    {-0.0011},
    {-0.0002},
    { 0.0008},
    {-0.0010},
    { 0.0013},
    {-0.0003},
    {-0.0001},
    {-0.0002},
    { 0.0001},
    { 0.0003},
    {-0.0012},
    { 0.0009},
    {-0.0003},
    { 0.0005},
    {-0.0018},
    { 0.0013},
    { 0.0004},
    { 0.0006},
    { 0.0003},
    {-0.0016},
    {-0.0005},
    { 0.0005},
    { 0.0004},
    { 0.0000},
    {-0.0007},
    {-0.0007},
    { 0.0008},
    {-0.0007},
    { 0.0008},
    { 0.0004},
    { 0.0003},
    { 0.0019},
    { 0.0003},
    {-0.0004},
    { 0.0021},
    {-0.0013},
    { 0.0006},
    {-0.0009},
    { 0.0012},
    { 0.0011},
    { 0.0003},
    { 0.0011},
    {-0.0003},
    {-0.0007},
    { 0.0001},
    { 0.0009},
    {-0.0002},
    {-0.0012},
    {-0.0010},
    {-0.0002},
    {-0.0007},
    {-0.0001},
    {-0.0008},
    { 0.0001},
    { 0.0001},
    { 0.0005},
    { 0.0007},
    {-0.0004},
    {-0.0007},
    {-0.0003},
    {-0.0005},
    { 0.0006},
    { 0.0001},
    {-0.0012},
    {-0.0004},
    { 0.0014},
    {-0.0005},
    { 0.0007},
    {-0.0001},
    {-0.0002},
    { 0.0010},
    {-0.0015},
    {-0.0003},
    { 0.0009},
    { 0.0000},
    {-0.0009},
    { 0.0010},
    { 0.0016},
    {-0.0002},
    {-0.0001},
    {-0.0004},
    {-0.0011},
    { 0.0004},
    { 0.0010},
    {-0.0007},
    {-0.0003},
    {-0.0008},
    {-0.0012},
    { 0.0003},
    { 0.0016},
    {-0.0019},
    {-0.0006},
    { 0.0006},
    { 0.0006},
    {-0.0002},
    {-0.0004},
    { 0.0003},
    { 0.0008},
    {-0.0013},
    {-0.0012},
    {-0.0002},
    {-0.0005},
    { 0.0010},
    {-0.0009},
    {-0.0003},
    { 0.0016},
    {-0.0013},
    { 0.0003},
    { 0.0003},
    {-0.0007},
    { 0.0016},
    { 0.0002},
    {-0.0013},
    {-0.0002},
    {-0.0009},
    {-0.0003},
    { 0.0010},
    { 0.0002},
    { 0.0013},
    {-0.0006},
    {-0.0006},
    { 0.0001},
    {-0.0005},
    { 0.0018},
    { 0.0002},
    { 0.0006},
    {-0.0002},
    {-0.0004},
    { 0.0000},
    { 0.0016},
    {-0.0011},
    {-0.0004},
    { 0.0002},
    { 0.0001},
    { 0.0025},
    {-0.0011},
    {-0.0008},
    { 0.0009},
    {-0.0001},
    {-0.0005},
    {-0.0001},
    { 0.0005},
    { 0.0001},
    {-0.0008},
    {-0.0001},
    { 0.0003},
    { 0.0008},
    {-0.0011},
    { 0.0003},
    { 0.0004},
    { 0.0005},
    { 0.0014},
    {-0.0002},
    { 0.0008},
    { 0.0004},
    { 0.0019},
    {-0.0005},
    {-0.0005},
    {-0.0005},
    { 0.0000},
    {-0.0006},
    { 0.0015},
    {-0.0011},
    {-0.0017},
    {-0.0002},
    { 0.0007},
    { 0.0004},
    {-0.0018},
    {-0.0001},
    {-0.0002},
    {-0.0001},
    { 0.0005},
    {-0.0003},
    { 0.0000},
    { 0.0007},
    { 0.0006},
    {-0.0014},
    { 0.0006},
    {-0.0003},
    { 0.0002},
    {-0.0002},
    {-0.0002},
    { 0.0002},
    {-0.0009},
    {-0.0001},
    {-0.0001},
    { 0.0000},
    { 0.0012},
    { 0.0000},
    { 0.0003},
    {-0.0008},
    { 0.0004},
    { 0.0000},
    {-0.0003},
    {-0.0013},
    {-0.0005},
    {-0.0005},
    {-0.0001},
    {-0.0004},
    { 0.0006},
    { 0.0000},
    {-0.0008},
    {-0.0004},
    {-0.0024},
    { 0.0003},
    {-0.0005},
    { 0.0007},
    { 0.0003},
    {-0.0004},
    {-0.0004},
    { 0.0003},
    { 0.0000},
    { 0.0014},
    {-0.0004},
    { 0.0000},
    {-0.0009},
    { 0.0012},
    {-0.0008},
    { 0.0028},
    { 0.0003},
    { 0.0001},
    { 0.0010},
    {-0.0005},
    { 0.0004},
    {-0.0005},
    { 0.0001},
    {-0.0005},
    { 0.0017},
    {-0.0012},
    {-0.0002},
    { 0.0010},
    {-0.0009},
    {-0.0001},
    { 0.0003},
    { 0.0001},
    { 0.0031},
    {-0.0005},
    { 0.0004},
    { 0.0002},
    {-0.0001},
    {-0.0003},
    {-0.0013},
    {-0.0015},
    {-0.0003},
    { 0.0010},
    { 0.0010},
    {-0.0002},
    { 0.0027},
    { 0.0004},
    { 0.0001},
    {-0.0007},
    { 0.0000},
    {-0.0002},
    { 0.0011},
    {-0.0009},
    {-0.0003},
    {-0.0006},
    { 0.0007},
    {-0.0015},
    { 0.0004},
    {-0.0002},
    { 0.0002},
    { 0.0001},
    {-0.0005},
    { 0.0000},
    { 0.0003},
    {-0.0011},
    { 0.0001},
    {-0.0001},
    { 0.0002},
    {-0.0006},
    { 0.0003},
    {-0.0004},
    {-0.0005},
    {-0.0005},
    { 0.0000},
    { 0.0007},
    {-0.0032},
    { 0.0001},
    { 0.0007},
    {-0.0008},
    {-0.0004},
    {-0.0004},
    {-0.0004},
    { 0.0010},
    {-0.0001},
    {-0.0001},
    { 0.0002},
    { 0.0000},
    { 0.0001},
    {-0.0009},
    {-0.0002},
    {-0.0009},
    {-0.0002},
    { 0.0002},
    { 0.0002},
    { 0.0010},
    { 0.0002},
    {-0.0005},
    {-0.0005},
    {-0.0021},
    { 0.0008},
    { 0.0000},
    { 0.0006},
    {-0.0006},
    {-0.0007},
    { 0.0007},
    {-0.0007},
    { 0.0005},
    { 0.0011},
    {-0.0006},
    {-0.0004},
    { 0.0012},
    { 0.0001},
    { 0.0010},
    {-0.0002},
    {-0.0004},
    {-0.0004},
    {-0.0003},
    {-0.0006},
    { 0.0011},
    { 0.0000},
    { 0.0012},
    { 0.0001},
    {-0.0003},
    { 0.0011},
    { 0.0007},
    {-0.0007},
    { 0.0005},
    {-0.0002},
    { 0.0008},
    { 0.0002},
    { 0.0008},
    {-0.0002},
    { 0.0005},
    { 0.0002},
    {-0.0009},
    { 0.0003},
    {-0.0008},
    { 0.0004},
    {-0.0009},
    { 0.0006},
    { 0.0011},
    { 0.0010},
    { 0.0006},
    {-0.0006},
    { 0.0005},
    { 0.0002},
    {-0.0035},
    { 0.0008},
    {-0.0006},
    {-0.0011},
    { 0.0008},
    {-0.0003},
    {-0.0005},
    { 0.0004},
    { 0.0000},
    { 0.0002},
    {-0.0011},
    { 0.0001},
    {-0.0009},
    { 0.0007},
    {-0.0003},
    { 0.0005},
    {-0.0005},
    { 0.0002},
    {-0.0021},
    {-0.0004},
    { 0.0000},
    { 0.0014},
    {-0.0004},
    {-0.0006},
    {-0.0031},
    {-0.0006},
    { 0.0004},
    { 0.0001},
    {-0.0012},
    {-0.0011},
    { 0.0004},
    {-0.0002},
    { 0.0000},
    {-0.0004},
    {-0.0002},
    { 0.0013},
    {-0.0007},
    {-0.0007},
    { 0.0001},
    { 0.0001},
    {-0.0010},
    { 0.0001},
    { 0.0006},
    {-0.0001},
    {-0.0008},
    {-0.0015},
    {-0.0008},
    {-0.0001},
    { 0.0010},
    {-0.0004},
    { 0.0000},
    {-0.0016},
    { 0.0000},
    { 0.0001},
    {-0.0004},
    {-0.0001},
    { 0.0014},
    {-0.0025},
    {-0.0006},
    {-0.0011},
    {-0.0002},
    {-0.0001},
    {-0.0005},
    {-0.0005},
    {-0.0014},
    { 0.0000},
    {-0.0007},
    { 0.0021},
    { 0.0002},
    {-0.0005},
    {-0.0009},
    {-0.0002},
    { 0.0020},
    { 0.0004},
    {-0.0011},
    {-0.0018},
    { 0.0005},
    { 0.0005},
    { 0.0000},
    { 0.0000},
    { 0.0004},
    { 0.0008},
    {-0.0004},
    {-0.0007},
    { 0.0001},
    { 0.0009},
    {-0.0001},
    { 0.0010},
    {-0.0005},
    {-0.0023},
    { 0.0004},
    { 0.0022},
    {-0.0013},
    { 0.0003},
    {-0.0003},
    {-0.0003},
    { 0.0010},
    {-0.0002},
    { 0.0000},
    { 0.0002},
    {-0.0004},
    {-0.0007},
    {-0.0005},
    {-0.0003},
    { 0.0005},
    {-0.0010},
    {-0.0004},
    { 0.0001},
    { 0.0003},
    { 0.0000},
    { 0.0008},
    { 0.0001},
    { 0.0004},
    { 0.0002},
    { 0.0001},
    {-0.0005},
    {-0.0005},
    {-0.0003},
    { 0.0016},
    { 0.0006},
    {-0.0014},
    { 0.0016},
    { 0.0016},
    { 0.0009},
    {-0.0002},
    {-0.0002},
    { 0.0005},
    {-0.0005},
    {-0.0004},
    {-0.0008},
    {-0.0003},
    {-0.0001},
    { 0.0005},
    {-0.0002},
    {-0.0004},
    {-0.0003},
    { 0.0003},
    { 0.0003},
    {-0.0003},
    { 0.0003},
    {-0.0004},
    { 0.0000},
    {-0.0008},
    {-0.0013},
    { 0.0007},
    {-0.0011},
    { 0.0015},
    {-0.0008},
    { 0.0007},
    {-0.0002},
    {-0.0004},
    { 0.0005},
    {-0.0008},
    { 0.0000},
    {-0.0008},
    { 0.0007},
    { 0.0007},
    { 0.0011},
    { 0.0000},
    { 0.0003},
    {-0.0005},
    { 0.0003},
    { 0.0012},
    { 0.0005},
    {-0.0006},
    {-0.0002},
    {-0.0007},
    {-0.0027},
    {-0.0012},
    {-0.0005},
    { 0.0005},
    { 0.0002},
    { 0.0001},
    { 0.0003},
    { 0.0003},
    { 0.0002},
    {-0.0002},
    {-0.0003},
    { 0.0009},
    { 0.0003},
    { 0.0003},
    { 0.0006},
    {-0.0002},
    {-0.0003},
    {-0.0004},
    { 0.0011},
    { 0.0000},
    { 0.0000},
    { 0.0000},
    {-0.0007},
    { 0.0001},
    { 0.0019},
    { 0.0011},
    {-0.0011},
    { 0.0002},
    { 0.0009},
    { 0.0002},
    { 0.0004},
    {-0.0001},
    {-0.0008},
    { 0.0001},
    { 0.0003},
    {-0.0002},
    {-0.0002},
    { 0.0002},
    { 0.0003},
    { 0.0000},
    { 0.0010},
    { 0.0013},
    {-0.0004},
    {-0.0009},
    {-0.0005},
    { 0.0018},
    { 0.0018},
    {-0.0006},
    {-0.0006},
    {-0.0009},
    { 0.0005},
    {-0.0004},
    {-0.0001},
    { 0.0012},
    {-0.0002},
    {-0.0006},
    {-0.0024},
    {-0.0002},
    {-0.0011},
    { 0.0003},
    { 0.0009},
    { 0.0002},
    { 0.0002},
    { 0.0003},
    {-0.0002},
    {-0.0002},
    { 0.0000},
    { 0.0000},
    {-0.0010},
    {-0.0007},
    { 0.0009},
    { 0.0001},
    { 0.0020},
    { 0.0001},
    { 0.0005},
    {-0.0003},
    { 0.0004},
    {-0.0005},
    { 0.0005},
    { 0.0002},
    { 0.0001},
    { 0.0001},
    { 0.0004},
    { 0.0008},
    { 0.0000},
    {-0.0008},
    { 0.0002},
    {-0.0012},
    {-0.0004},
    { 0.0004},
    {-0.0001},
    { 0.0010},
    {-0.0009},
    {-0.0007},
    {-0.0006},
    {-0.0002},
    { 0.0001},
    {-0.0004},
    { 0.0002},
    { 0.0000},
    { 0.0000},
    {-0.0012},
    {-0.0002},
    {-0.0002},
    { 0.0009},
    {-0.0008},
    {-0.0001},
    { 0.0008},
    {-0.0001},
    {-0.0002},
    {-0.0004},
    { 0.0009},
    { 0.0001},
    { 0.0005},
    { 0.0015},
    {-0.0001},
    {-0.0002},
    {-0.0004},
    { 0.0009},
    {-0.0010},
    {-0.0005},
    { 0.0004},
    { 0.0011},
    { 0.0009},
    {-0.0003},
    {-0.0002},
    { 0.0006},
    { 0.0003},
    {-0.0003},
    {-0.0002},
    { 0.0015},
    { 0.0003},
    { 0.0001},
    {-0.0001},
    {-0.0001},
    {-0.0005},
    {-0.0001},
    { 0.0003},
    { 0.0002},
    { 0.0001},
    {-0.0002},
    {-0.0008},
    { 0.0003},
    {-0.0001},
    { 0.0000},
    { 0.0002},
    {-0.0002},
    {-0.0001},
    {-0.0005},
    { 0.0002},
    { 0.0000},
    { 0.0000},
    { 0.0004},
    { 0.0000},
    {-0.0013},
    {-0.0043},
    { 0.0007},
    { 0.0001},
    {-0.0007},
    {-0.0001},
    { 0.0003},
    { 0.0013},
    { 0.0005},
    {-0.0007},
    { 0.0002},
    {-0.0014},
    {-0.0002},
    { 0.0000},
    { 0.0020},
    { 0.0009},
    {-0.0018},
    {-0.0005},
    {-0.0009},
    {-0.0005},
    { 0.0008},
    { 0.0005},
    {-0.0004},
    { 0.0004},
    { 0.0001},
    { 0.0003},
    { 0.0003},
    {-0.0008},
    {-0.0007},
    { 0.0008},
    {-0.0004},
    {-0.0028},
    {-0.0006},
    {-0.0022},
    {-0.0009},
    { 0.0003},
    { 0.0003},
    { 0.0000},
    { 0.0021},
    {-0.0017},
    { 0.0009},
    {-0.0001},
    {-0.0005},
    {-0.0015},
    { 0.0009},
    { 0.0009},
    {-0.0009},
    { 0.0010},
    { 0.0000},
    { 0.0018},
    {-0.0011},
    { 0.0001},
    {-0.0008},
    { 0.0009},
    { 0.0022},
    { 0.0011},
    {-0.0002},
    {-0.0018},
    { 0.0019},
    {-0.0004},
    {-0.0004},
    { 0.0002},
    { 0.0027},
    {-0.0012},
    {-0.0003},
    {-0.0002},
    { 0.0013},
    { 0.0002},
    {-0.0003},
    { 0.0002},
    { 0.0003},
    {-0.0002},
    { 0.0020},
    {-0.0003},
    { 0.0002},
    {-0.0001},
    {-0.0001},
    { 0.0006},
    {-0.0012},
    { 0.0011},
    { 0.0001},
    { 0.0001},
    { 0.0004},
    {-0.0002},
    { 0.0029},
    {-0.0016},
    { 0.0005},
    {-0.0029},
    {-0.0002},
    { 0.0013},
    { 0.0011},
    {-0.0003},
    { 0.0002},
    { 0.0001},
    { 0.0001},
    {-0.0005},
    { 0.0000},
    { 0.0009},
    { 0.0016},
    { 0.0005},
    { 0.0003},
    { 0.0004},
    {-0.0001},
    { 0.0020},
    { 0.0006},
    {-0.0006},
    {-0.0004},
    { 0.0012},
    { 0.0012},
    { 0.0017},
    { 0.0017},
    {-0.0004},
    { 0.0001},
    { 0.0001},
    {-0.0006},
    {-0.0003},
    { 0.0004},
    { 0.0008},
    {-0.0014},
    { 0.0011},
    {-0.0003},
    { 0.0004},
    {-0.0001},
    { 0.0017},
    { 0.0008},
    { 0.0005},
    {-0.0002},
    { 0.0001},
    {-0.0013},
    { 0.0009},
    {-0.0008},
    {-0.0003},
    { 0.0013},
    {-0.0002},
    {-0.0004},
    { 0.0007},
    {-0.0009},
    { 0.0002},
    {-0.0002},
    {-0.0006},
    { 0.0003},
    {-0.0003},
    {-0.0004},
    {-0.0006},
    {-0.0004},
    { 0.0006},
    {-0.0006},
    { 0.0019},
    {-0.0012},
    {-0.0002},
    { 0.0003},
    { 0.0001},
    { 0.0001},
    { 0.0002},
    { 0.0004},
    { 0.0000},
    {-0.0013},
    {-0.0017},
    {-0.0017},
    {-0.0006},
    {-0.0003},
    { 0.0006},
    { 0.0001},
    { 0.0007},
    {-0.0004},
    { 0.0000},
    { 0.0019},
    {-0.0008},
    {-0.0006},
    { 0.0003},
    {-0.0004},
    {-0.0002},
    { 0.0008},
    {-0.0007},
    {-0.0004},
    {-0.0001},
    {-0.0004},
    {-0.0003},
    { 0.0004},
    {-0.0008},
    { 0.0015},
    { 0.0011},
    {-0.0013},
    {-0.0003},
    { 0.0000},
    { 0.0006},
    {-0.0003},
    {-0.0003},
    { 0.0005},
    {-0.0016},
    { 0.0005},
    {-0.0002},
    {-0.0009},
    {-0.0001},
    {-0.0004},
    { 0.0007},
    { 0.0004},
    { 0.0015},
    { 0.0005},
    {-0.0007},
    { 0.0002},
    {-0.0007},
    {-0.0009},
    {-0.0029},
    { 0.0003},
    {-0.0002},
    {-0.0010},
    {-0.0001},
    { 0.0005},
    {-0.0003},
    { 0.0002},
    {-0.0006},
    { 0.0008},
    {-0.0010},
    { 0.0005},
    { 0.0003},
    { 0.0009},
    { 0.0002},
    { 0.0001},
    { 0.0002},
    {-0.0010},
    {-0.0017},
    { 0.0004},
    {-0.0007},
    { 0.0019},
    {-0.0001},
    {-0.0004},
    {-0.0023},
    { 0.0015},
    { 0.0009},
    {-0.0006},
    {-0.0001},
    { 0.0000},
    { 0.0001},
    { 0.0003},
    {-0.0002},
    { 0.0039},
    {-0.0007},
    { 0.0000},
    {-0.0002},
    {-0.0005},
    { 0.0007},
    { 0.0010},
    {-0.0004},
    { 0.0006},
    {-0.0003},
    {-0.0006},
    { 0.0003},
    { 0.0022},
    {-0.0030},
    { 0.0010},
    { 0.0005},
    { 0.0000},
    {-0.0005},
    {-0.0004},
    {-0.0010},
    {-0.0001},
    { 0.0000},
    {-0.0002},
    { 0.0003},
    {-0.0003},
    {-0.0007},
    {-0.0001},
    { 0.0016},
    {-0.0009},
    { 0.0001},
    { 0.0005},
    { 0.0016},
    {-0.0013},
    {-0.0003},
    {-0.0002},
    { 0.0014},
    {-0.0008},
    { 0.0026},
    {-0.0003},
    { 0.0007},
    { 0.0011},
    { 0.0005},
    {-0.0001},
    { 0.0004},
    {-0.0003},
    { 0.0006},
    {-0.0013},
    { 0.0002},
    {-0.0003},
    {-0.0010},
    { 0.0000},
    {-0.0007},
    {-0.0001},
    { 0.0008},
    {-0.0010},
    {-0.0005},
    { 0.0010},
    { 0.0013},
    { 0.0008},
    {-0.0004},
    { 0.0020},
    {-0.0002},
    { 0.0005},
    { 0.0003},
    { 0.0004},
    { 0.0005},
    { 0.0007},
    {-0.0008},
    { 0.0003},
    {-0.0008},
    { 0.0011},
    { 0.0005},
    { 0.0002},
    {-0.0005},
    {-0.0011},
    { 0.0001},
    {-0.0002},
    {-0.0003},
    { 0.0016},
    {-0.0005},
    {-0.0002},
    {-0.0008},
    { 0.0023},
    { 0.0015},
    { 0.0001},
    {-0.0004},
    { 0.0015},
    {-0.0002},
    { 0.0003},
    { 0.0001},
    { 0.0002},
    {-0.0007},
    { 0.0005},
    {-0.0001},
    { 0.0007},
    {-0.0007},
    { 0.0018},
    {-0.0002},
    { 0.0005},
    {-0.0002},
    {-0.0004},
    {-0.0001},
    {-0.0001},
    { 0.0002},
    { 0.0009},
    { 0.0003},
    {-0.0006},
    {-0.0003},
    {-0.0022},
    { 0.0002},
    { 0.0004},
    { 0.0004},
    {-0.0005},
    {-0.0008},
    {-0.0001},
    { 0.0000},
    { 0.0010},
    { 0.0001},
    {-0.0005},
    {-0.0003},
    {-0.0005},
    { 0.0005},
    {-0.0003},
    { 0.0000},
    { 0.0002},
    { 0.0001},
    {-0.0014},
    {-0.0005},
    {-0.0001},
    { 0.0006},
    { 0.0004},
    { 0.0005},
    { 0.0017},
    { 0.0009},
    { 0.0008},
    { 0.0017},
    { 0.0006},
    {-0.0003},
    { 0.0004},
    { 0.0000},
    {-0.0015},
    {-0.0001},
    { 0.0000},
    {-0.0008},
    { 0.0000},
    { 0.0004},
    {-0.0006},
    { 0.0001},
    { 0.0001},
    { 0.0003},
    {-0.0001},
    { 0.0001},
    {-0.0011},
    {-0.0011},
    { 0.0010},
    { 0.0004},
    { 0.0008},
    { 0.0007},
    { 0.0004},
    {-0.0023},
    { 0.0023},
    {-0.0001},
    {-0.0004},
    { 0.0000},
    {-0.0003},
    {-0.0008},
    { 0.0005},
    { 0.0004},
    {-0.0004},
    { 0.0001},
    {-0.0007},
    {-0.0001},
    {-0.0006},
    {-0.0012},
    {-0.0027},
    { 0.0002},
    {-0.0012},
    { 0.0013},
    { 0.0000},
    {-0.0008},
    { 0.0009},
    {-0.0015},
    { 0.0007},
    { 0.0002},
    { 0.0001},
    {-0.0013},
    { 0.0002},
    {-0.0004},
    {-0.0002},
    {-0.0027},
    { 0.0013},
    {-0.0005},
    { 0.0006},
    { 0.0000},
    {-0.0005},
    { 0.0011},
    { 0.0000},
    {-0.0003},
    { 0.0018},
    {-0.0003},
    { 0.0006},
    {-0.0008},
    {-0.0006},
    {-0.0015},
    {-0.0009},
    {-0.0017},
    { 0.0005},
    { 0.0007},
    {-0.0006},
    {-0.0004},
    { 0.0016},
    { 0.0000},
    { 0.0003},
    {-0.0019},
    { 0.0003},
    { 0.0005},
    { 0.0003},
    { 0.0001},
    {-0.0002},
    { 0.0007},
    { 0.0001},
    {-0.0012},
    { 0.0008},
    {-0.0011},
    { 0.0008},
    { 0.0010},
    {-0.0003},
    {-0.0003},
    {-0.0006},
    {-0.0004},
    { 0.0000},
    { 0.0011},
    { 0.0003},
    {-0.0005},
    {-0.0002},
    { 0.0001},
    {-0.0004},
    { 0.0008},
    {-0.0007},
    {-0.0005},
    {-0.0005},
    {-0.0003},
    {-0.0005},
    { 0.0006},
    {-0.0014},
    {-0.0001},
    { 0.0003},
    { 0.0009},
    { 0.0012},
    {-0.0023},
    {-0.0004},
    { 0.0002},
    { 0.0001},
    {-0.0019},
    { 0.0012},
    {-0.0022},
    { 0.0001},
    {-0.0004},
    {-0.0001},
    { 0.0012},
    { 0.0001},
    { 0.0000},
    { 0.0004},
    {-0.0008},
    { 0.0006},
    {-0.0007},
    { 0.0008},
    {-0.0009},
    {-0.0005},
    {-0.0001},
    {-0.0004},
    { 0.0001},
    {-0.0005},
    {-0.0004},
    {-0.0012},
    { 0.0002},
    { 0.0011},
    { 0.0008},
    {-0.0009},
    { 0.0010},
    {-0.0003},
    {-0.0006},
    { 0.0005},
    { 0.0002},
    {-0.0003},
    { 0.0016},
    {-0.0006},
    { 0.0000},
    { 0.0008},
    {-0.0002},
    { 0.0000},
    { 0.0000},
    { 0.0003},
    {-0.0011},
    {-0.0009},
    {-0.0013},
    {-0.0003},
    { 0.0009},
    { 0.0007},
    {-0.0005},
    { 0.0012},
    {-0.0009},
    { 0.0004},
    {-0.0008},
    {-0.0011},
    { 0.0013},
    { 0.0013},
    { 0.0005 }};

    private static Dataset defaultDataset;
    private static IEnumerable<string> defaultAllowedInputVariables;
    private static string defaultPriceVariable;

    static TradingProblemData() {
      defaultDataset = new Dataset(new string[] { "AUD" }, audInUsd);
      defaultDataset.Name = "AUD in USD";
      defaultDataset.Description = "Price of Australian dollar in US dollar.";
      defaultAllowedInputVariables = new List<string>() { "AUD" };
      defaultPriceVariable = "AUD";
    }
    #endregion

    public IValueParameter<StringValue> PriceVariableParameter {
      get { return (IValueParameter<StringValue>)Parameters[PriceVariableParameterName]; }
    }
    public IValueParameter<DoubleValue> TransactionCostsParameter {
      get { return (IValueParameter<DoubleValue>)Parameters[TransactionCostsParameterName]; }
    }
    public string PriceVariable {
      get { return PriceVariableParameter.Value.Value; }
    }
    public double TransactionCosts {
      get { return TransactionCostsParameter.Value.Value; }
    }

    [StorableConstructor]
    private TradingProblemData(bool deserializing) : base(deserializing) { }
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      RegisterParameterEvents();
    }

    private TradingProblemData(TradingProblemData original, Cloner cloner)
      : base(original, cloner) {
      RegisterParameterEvents();
    }
    public override IDeepCloneable Clone(Cloner cloner) { return new TradingProblemData(this, cloner); }

    public TradingProblemData()
      : this(defaultDataset, defaultAllowedInputVariables, defaultPriceVariable) {
    }

    public TradingProblemData(Dataset dataset, IEnumerable<string> allowedInputVariables, string targetVariable)
      : base(dataset, allowedInputVariables) {
      var variables = InputVariables.Select(x => x.AsReadOnly()).ToList();
      Parameters.Add(new ConstrainedValueParameter<StringValue>(PriceVariableParameterName, new ItemSet<StringValue>(variables), variables.Where(x => x.Value == targetVariable).First()));
      Parameters.Add(new FixedValueParameter<DoubleValue>(TransactionCostsParameterName, "The absolute cost of on buy/sell transaction (assumed to be constant and independent of transaction volume)", new DoubleValue(0.0002)));
      RegisterParameterEvents();
    }

    private void RegisterParameterEvents() {
      PriceVariableParameter.ValueChanged += new EventHandler(PriceVariableParameter_ValueChanged);
      TransactionCostsParameter.Value.ValueChanged += new EventHandler(TransactionCostsParameter_ValueChanged);
    }

    private void TransactionCostsParameter_ValueChanged(object sender, EventArgs e) {
      OnChanged();
    }
    private void PriceVariableParameter_ValueChanged(object sender, EventArgs e) {
      OnChanged();
    }

    #region Import from file
    public static TradingProblemData ImportFromFile(string fileName) {
      TableFileParser csvFileParser = new TableFileParser();
      csvFileParser.Parse(fileName);

      Dataset dataset = new Dataset(csvFileParser.VariableNames, csvFileParser.Values);
      dataset.Name = Path.GetFileName(fileName);

      TradingProblemData problemData = new TradingProblemData(dataset, dataset.VariableNames.Skip(1), dataset.VariableNames.First());
      problemData.Name = "Data imported from " + Path.GetFileName(fileName);
      return problemData;
    }
    #endregion
  }
}
